(* Zentrale Datenstruktur Goal 
 * beinhaltet :
 * - aktive Goals ( Formeln, die es zu beweisen gilt )
 * - Variablenumgebungen ( Env und Gamma Env Pointer )
 * - Definitionen Hashtable
 *)


signature Goal =
sig
    type goal

(* ------------ Methoden FIRST Goal ---------------- *)

(* Returns a initial empty goal *)
    val init_goal : unit -> goal

(* Returns a initial goal with envs from oldgoal *)
    val init_goal_env: goal -> goal 

(* Adds a new goal to list *)
    val first_goal : goal * Data.sourceexpr -> goal

(* Returns the active goal in queue *)
    val get_first_goal : goal -> Data.pter

(* Replace the first/active goal by an Preterm *)
    val update_first_goal : goal * Data.pter -> goal

(* Removes the first goal from list *)
    val remove_first_goal : goal -> goal

(* Typecheck of the first goal *)
    val typecheck : goal -> bool

(* ---------- Methoden GOAL Erweiterung ------------- *)

(* splits a goal and takes the env of the father *)
    val split_goal : goal * Data.pter -> goal 

(* adds a goal *)
    val add_goal : goal * goal -> goal 

(* Updates the Constant type environment *)
    val update_ctypeenv : goal * string * Data.ty -> goal

(* Updates the Free Variable type environment *)
    val update_ftypeenv : goal * string * Data.ty -> goal

(* Adds a new bound Variable in env used in Equality.sml *)
    val update_bndvar : goal * string -> goal

(* Updates the typelist, new types will be declared *)
    val new_type : goal * string -> goal

(* Checks, whether some free variables are catched by bounded ones *)
    val kaper_check : goal -> goal

(* Is called, if Intro/Elim introduces a new Free Variable. *) 
    val freeVarIntro : goal * int -> string

(* ----------------- Gamma Methoden ----------------- *)

(* Returns the next free Gamma Nr. *)
    val next_gamma_nr : goal -> int

(* Adds an Gamma Variable to the active goal *)
    val add_gamma : goal -> goal

(* Adds an Delta Variable to the active goal *)
    val add_delta : goal * int -> goal

(* Adds an Delta Variable to the active goal *)
    val substitute_gamma : goal -> (int * Data.pter list) -> bool

(* Merge the two goals' environment *)
    val merge_envs : goal * goal -> goal


(* ----------------- Definition Methoden  ------------ *)

(* Adds a new definition        name         term          CLASSICAL|LEMMA|AXIOM *)
    val new_definition : goal * string * Data.sourceexpr * Data.kind_of_def -> goal    
(* Saves the solved goal as an axiom under string *)
    val save_goal : goal * string * Data.pter -> goal

(* Checks if the term pter contains some definitions *)
    val check_def : goal * Data.pter -> bool

(* Realises a translation of definitions contained in pter *)
    val translate_def : goal * Data.pter -> Data.pter

(* Returns the pter which is defined for a string *)
    val get_definition : goal * string -> Data.pter

(* Returns the kind of definition which is defined for a string *)
    val get_definitions_kind : goal * string -> Data.kind_of_def

(* Changes the kind of definition *)
    val change_definitions_kind : goal * string * Data.kind_of_def -> unit

(* Returns the Reference of an old environment *)
    val get_envref : goal * string -> Env.env ref

(* ---------------- Tunnel Functions (ENV) ----------- *)

(* Get the map from Env.freemaps2i *)
    val freemaps2i : goal -> ( string -> int)

(* Get the map from Env.freemaps2i *)
    val freemapi2s : goal -> ( int -> string)

(* Get the map from Env.bndmapi2s *)
    val bndmapi2s : goal -> ( int -> string )

(* Get the map from Env.bndmaps2i *)
    val bndmaps2i : goal -> ( string -> int )

(* Get a typ mapping : string -> Data.ty *)
    val typc   : goal -> (string -> Data.ty)

(* Get a typ mapping : int -> Data.ty *)
    val typi   : goal -> ( int -> Data.ty )

(* Get a typlist mapping : string -> bool *)
    val typemap : goal -> (string -> bool)

(* Get the nr. of known free Variables *)
    val nr_freevars: goal -> int

(* Get the nr. of known bounded Variables *)
    val nr_bndvars: goal -> int

(* ----------------- Zusatzmethoden ----------------- *)

(* Returns the active Environment *)
    val get_env : goal -> Env.env

(* Sets a new Environment *)
    val set_env : goal * Env.env -> goal

(* Shifts the pters and envs cyclic up *)
    val cyclic_shift : goal -> goal

(* Returns the Numbers of active goals and subgoals *)
    val get_nr_of_goals : goal -> int

(* Print the goals *)
    val print_goal  : goal -> unit

(* Latexprint the pters *)
    val latex_goal  : goal * Data.pter -> string

end